<?php
/*
 * Copyright 2021 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * GENERATED CODE WARNING
 * Generated by gapic-generator-php from the file
 * https://github.com/google/googleapis/blob/master/google/ads/googleads/v9/services/smart_campaign_suggest_service.proto
 * Updates to the above are reflected here through a refresh process.
 */

namespace Google\Ads\GoogleAds\V9\Services\Gapic;

use Google\Ads\GoogleAds\V9\Services\SmartCampaignSuggestionInfo;
use Google\Ads\GoogleAds\V9\Services\SuggestKeywordThemesRequest;
use Google\Ads\GoogleAds\V9\Services\SuggestKeywordThemesResponse;

use Google\Ads\GoogleAds\V9\Services\SuggestSmartCampaignAdRequest;
use Google\Ads\GoogleAds\V9\Services\SuggestSmartCampaignAdResponse;
use Google\Ads\GoogleAds\V9\Services\SuggestSmartCampaignBudgetOptionsRequest;
use Google\Ads\GoogleAds\V9\Services\SuggestSmartCampaignBudgetOptionsRequest\SuggestionDataOneof;
use Google\Ads\GoogleAds\V9\Services\SuggestSmartCampaignBudgetOptionsResponse;
use Google\ApiCore\ApiException;
use Google\ApiCore\CredentialsWrapper;
use Google\ApiCore\GapicClientTrait;
use Google\ApiCore\PathTemplate;
use Google\ApiCore\RequestParamsHeaderDescriptor;
use Google\ApiCore\RetrySettings;
use Google\ApiCore\Transport\TransportInterface;
use Google\ApiCore\ValidationException;
use Google\Auth\FetchAuthTokenInterface;

/**
 * Service Description: Service to get suggestions for Smart Campaigns.
 *
 * This class provides the ability to make remote calls to the backing service through method
 * calls that map to API methods. Sample code to get started:
 *
 * ```
 * $smartCampaignSuggestServiceClient = new SmartCampaignSuggestServiceClient();
 * try {
 *     $customerId = 'customer_id';
 *     $suggestionInfo = new SmartCampaignSuggestionInfo();
 *     $response = $smartCampaignSuggestServiceClient->suggestKeywordThemes($customerId, $suggestionInfo);
 * } finally {
 *     $smartCampaignSuggestServiceClient->close();
 * }
 * ```
 *
 * Many parameters require resource names to be formatted in a particular way. To
 * assist with these names, this class includes a format method for each type of
 * name, and additionally a parseName method to extract the individual identifiers
 * contained within formatted names that are returned by the API.
 */
class SmartCampaignSuggestServiceGapicClient
{
    use GapicClientTrait;

    /**
     * The name of the service.
     */
    const SERVICE_NAME = 'google.ads.googleads.v9.services.SmartCampaignSuggestService';

    /**
     * The default address of the service.
     */
    const SERVICE_ADDRESS = 'googleads.googleapis.com';

    /**
     * The default port of the service.
     */
    const DEFAULT_SERVICE_PORT = 443;

    /**
     * The name of the code generator, to be included in the agent header.
     */
    const CODEGEN_NAME = 'gapic';

    /**
     * The default scopes required by the service.
     */
    public static $serviceScopes = [
        'https://www.googleapis.com/auth/adwords',
    ];

    private static $campaignNameTemplate;

    private static $pathTemplateMap;

    private static function getClientDefaults()
    {
        return [
            'serviceName' => self::SERVICE_NAME,
            'serviceAddress' => self::SERVICE_ADDRESS . ':' . self::DEFAULT_SERVICE_PORT,
            'clientConfig' => __DIR__ . '/../resources/smart_campaign_suggest_service_client_config.json',
            'descriptorsConfigPath' => __DIR__ . '/../resources/smart_campaign_suggest_service_descriptor_config.php',
            'gcpApiConfigPath' => __DIR__ . '/../resources/smart_campaign_suggest_service_grpc_config.json',
            'credentialsConfig' => [
                'defaultScopes' => self::$serviceScopes,
            ],
            'transportConfig' => [
                'rest' => [
                    'restClientConfigPath' => __DIR__ . '/../resources/smart_campaign_suggest_service_rest_client_config.php',
                ],
            ],
        ];
    }

    private static function getCampaignNameTemplate()
    {
        if (self::$campaignNameTemplate == null) {
            self::$campaignNameTemplate = new PathTemplate('customers/{customer_id}/campaigns/{campaign_id}');
        }

        return self::$campaignNameTemplate;
    }

    private static function getPathTemplateMap()
    {
        if (self::$pathTemplateMap == null) {
            self::$pathTemplateMap = [
                'campaign' => self::getCampaignNameTemplate(),
            ];
        }

        return self::$pathTemplateMap;
    }

    /**
     * Formats a string containing the fully-qualified path to represent a campaign
     * resource.
     *
     * @param string $customerId
     * @param string $campaignId
     *
     * @return string The formatted campaign resource.
     */
    public static function campaignName($customerId, $campaignId)
    {
        return self::getCampaignNameTemplate()->render([
            'customer_id' => $customerId,
            'campaign_id' => $campaignId,
        ]);
    }

    /**
     * Parses a formatted name string and returns an associative array of the components in the name.
     * The following name formats are supported:
     * Template: Pattern
     * - campaign: customers/{customer_id}/campaigns/{campaign_id}
     *
     * The optional $template argument can be supplied to specify a particular pattern,
     * and must match one of the templates listed above. If no $template argument is
     * provided, or if the $template argument does not match one of the templates
     * listed, then parseName will check each of the supported templates, and return
     * the first match.
     *
     * @param string $formattedName The formatted name string
     * @param string $template      Optional name of template to match
     *
     * @return array An associative array from name component IDs to component values.
     *
     * @throws ValidationException If $formattedName could not be matched.
     */
    public static function parseName($formattedName, $template = null)
    {
        $templateMap = self::getPathTemplateMap();
        if ($template) {
            if (!isset($templateMap[$template])) {
                throw new ValidationException("Template name $template does not exist");
            }

            return $templateMap[$template]->match($formattedName);
        }

        foreach ($templateMap as $templateName => $pathTemplate) {
            try {
                return $pathTemplate->match($formattedName);
            } catch (ValidationException $ex) {
                // Swallow the exception to continue trying other path templates
            }
        }

        throw new ValidationException("Input did not match any known format. Input: $formattedName");
    }

    /**
     * Constructor.
     *
     * @param array $options {
     *     Optional. Options for configuring the service API wrapper.
     *
     *     @type string $serviceAddress
     *           The address of the API remote host. May optionally include the port, formatted
     *           as "<uri>:<port>". Default 'googleads.googleapis.com:443'.
     *     @type string|array|FetchAuthTokenInterface|CredentialsWrapper $credentials
     *           The credentials to be used by the client to authorize API calls. This option
     *           accepts either a path to a credentials file, or a decoded credentials file as a
     *           PHP array.
     *           *Advanced usage*: In addition, this option can also accept a pre-constructed
     *           {@see \Google\Auth\FetchAuthTokenInterface} object or
     *           {@see \Google\ApiCore\CredentialsWrapper} object. Note that when one of these
     *           objects are provided, any settings in $credentialsConfig will be ignored.
     *     @type array $credentialsConfig
     *           Options used to configure credentials, including auth token caching, for the
     *           client. For a full list of supporting configuration options, see
     *           {@see \Google\ApiCore\CredentialsWrapper::build()} .
     *     @type bool $disableRetries
     *           Determines whether or not retries defined by the client configuration should be
     *           disabled. Defaults to `false`.
     *     @type string|array $clientConfig
     *           Client method configuration, including retry settings. This option can be either
     *           a path to a JSON file, or a PHP array containing the decoded JSON data. By
     *           default this settings points to the default client config file, which is
     *           provided in the resources folder.
     *     @type string|TransportInterface $transport
     *           The transport used for executing network requests. May be either the string
     *           `rest` or `grpc`. Defaults to `grpc` if gRPC support is detected on the system.
     *           *Advanced usage*: Additionally, it is possible to pass in an already
     *           instantiated {@see \Google\ApiCore\Transport\TransportInterface} object. Note
     *           that when this object is provided, any settings in $transportConfig, and any
     *           $serviceAddress setting, will be ignored.
     *     @type array $transportConfig
     *           Configuration options that will be used to construct the transport. Options for
     *           each supported transport type should be passed in a key for that transport. For
     *           example:
     *           $transportConfig = [
     *               'grpc' => [...],
     *               'rest' => [...],
     *           ];
     *           See the {@see \Google\ApiCore\Transport\GrpcTransport::build()} and
     *           {@see \Google\ApiCore\Transport\RestTransport::build()} methods for the
     *           supported options.
     *     @type callable $clientCertSource
     *           A callable which returns the client cert as a string. This can be used to
     *           provide a certificate and private key to the transport layer for mTLS.
     * }
     *
     * @throws ValidationException
     */
    public function __construct(array $options = [])
    {
        $clientOptions = $this->buildClientOptions($options);
        $this->setClientOptions($clientOptions);
    }

    /**
     * Suggests keyword themes to advertise on.
     *
     * Sample code:
     * ```
     * $smartCampaignSuggestServiceClient = new SmartCampaignSuggestServiceClient();
     * try {
     *     $customerId = 'customer_id';
     *     $suggestionInfo = new SmartCampaignSuggestionInfo();
     *     $response = $smartCampaignSuggestServiceClient->suggestKeywordThemes($customerId, $suggestionInfo);
     * } finally {
     *     $smartCampaignSuggestServiceClient->close();
     * }
     * ```
     *
     * @param string                      $customerId     Required. The ID of the customer.
     * @param SmartCampaignSuggestionInfo $suggestionInfo Required. Information to get keyword theme suggestions.
     *                                                    Required fields:
     *                                                    * suggestion_info.final_url
     *                                                    * suggestion_info.language_code
     *                                                    * suggestion_info.geo_target
     *
     *                                                    Recommended fields:
     *                                                    * suggestion_info.business_setting
     * @param array                       $optionalArgs   {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a
     *           {@see Google\ApiCore\RetrySettings} object, or an associative array of retry
     *           settings parameters. See the documentation on
     *           {@see Google\ApiCore\RetrySettings} for example usage.
     * }
     *
     * @return \Google\Ads\GoogleAds\V9\Services\SuggestKeywordThemesResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function suggestKeywordThemes($customerId, $suggestionInfo, array $optionalArgs = [])
    {
        $request = new SuggestKeywordThemesRequest();
        $requestParamHeaders = [];
        $request->setCustomerId($customerId);
        $request->setSuggestionInfo($suggestionInfo);
        $requestParamHeaders['customer_id'] = $customerId;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('SuggestKeywordThemes', SuggestKeywordThemesResponse::class, $optionalArgs, $request)->wait();
    }

    /**
     * Suggests a Smart campaign ad compatible with the Ad family of resources,
     * based on data points such as targeting and the business to advertise.
     *
     * Sample code:
     * ```
     * $smartCampaignSuggestServiceClient = new SmartCampaignSuggestServiceClient();
     * try {
     *     $customerId = 'customer_id';
     *     $suggestionInfo = new SmartCampaignSuggestionInfo();
     *     $response = $smartCampaignSuggestServiceClient->suggestSmartCampaignAd($customerId, $suggestionInfo);
     * } finally {
     *     $smartCampaignSuggestServiceClient->close();
     * }
     * ```
     *
     * @param string                      $customerId     Required. The ID of the customer.
     * @param SmartCampaignSuggestionInfo $suggestionInfo Required. Inputs used to suggest a Smart campaign ad.
     *                                                    Required fields: final_url, language_code, keyword_themes.
     *                                                    Optional but recommended fields to improve the quality of the suggestion:
     *                                                    business_setting and geo_target.
     * @param array                       $optionalArgs   {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a
     *           {@see Google\ApiCore\RetrySettings} object, or an associative array of retry
     *           settings parameters. See the documentation on
     *           {@see Google\ApiCore\RetrySettings} for example usage.
     * }
     *
     * @return \Google\Ads\GoogleAds\V9\Services\SuggestSmartCampaignAdResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function suggestSmartCampaignAd($customerId, $suggestionInfo, array $optionalArgs = [])
    {
        $request = new SuggestSmartCampaignAdRequest();
        $requestParamHeaders = [];
        $request->setCustomerId($customerId);
        $request->setSuggestionInfo($suggestionInfo);
        $requestParamHeaders['customer_id'] = $customerId;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('SuggestSmartCampaignAd', SuggestSmartCampaignAdResponse::class, $optionalArgs, $request)->wait();
    }

    /**
     * Returns BudgetOption suggestions.
     *
     * Sample code:
     * ```
     * $smartCampaignSuggestServiceClient = new SmartCampaignSuggestServiceClient();
     * try {
     *     $customerId = 'customer_id';
     *     $suggestionData = new SuggestionDataOneof();
     *     $suggestionData->setCampaign('formattedCampaign-1139828244');
     *     $response = $smartCampaignSuggestServiceClient->suggestSmartCampaignBudgetOptions($customerId, $suggestionData);
     * } finally {
     *     $smartCampaignSuggestServiceClient->close();
     * }
     * ```
     *
     * @param string              $customerId     Required. The ID of the customer whose budget options are to be suggested.
     * @param SuggestionDataOneof $suggestionData An instance of the wrapper class for the required proto oneof suggestion_data.
     * @param array               $optionalArgs   {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a
     *           {@see Google\ApiCore\RetrySettings} object, or an associative array of retry
     *           settings parameters. See the documentation on
     *           {@see Google\ApiCore\RetrySettings} for example usage.
     * }
     *
     * @return \Google\Ads\GoogleAds\V9\Services\SuggestSmartCampaignBudgetOptionsResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function suggestSmartCampaignBudgetOptions($customerId, $suggestionData, array $optionalArgs = [])
    {
        $request = new SuggestSmartCampaignBudgetOptionsRequest();
        $requestParamHeaders = [];
        $request->setCustomerId($customerId);
        if ($suggestionData->isCampaign()) {
            $request->setCampaign($suggestionData->getCampaign());
        } elseif ($suggestionData->isSuggestionInfo()) {
            $request->setSuggestionInfo($suggestionData->getSuggestionInfo());
        } else {
            throw new ValidationException("A field for the oneof suggestion_data must be set in param $suggestionData");
        }

        
        $requestParamHeaders['customer_id'] = $customerId;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('SuggestSmartCampaignBudgetOptions', SuggestSmartCampaignBudgetOptionsResponse::class, $optionalArgs, $request)->wait();
    }
}
